@@ -222,6 +222,7 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 		{ "MODEL_DTU710",  "Wacom PL710",    9, 0x39 },
 		{ "MODEL_DTF720",  "Wacom DTF720",  10, 0xC0 },
 		{ "MODEL_DTF521",  "Wacom DTF521",  11, 0xC4 },
+		{ "MODEL_DTU1931", "Wacom DTU1931", 12, 0xC7 },
 		{ NULL }
 	};
 
@@ -257,9 +258,10 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 
 	static USBSUBTYPE xBamboo[] =
 	{
-		{ "MTE_450", "Wacom Bamboo", 1, 0x65 },
-		{ "CTE_450", "Wacom BambooFun 4x5", 2, 0x17 },
-		{ "CTE_650", "Wacom BambooFun 6x8", 3, 0x18 },
+		{ "MTE_450", "Wacom Bamboo",         1, 0x65 },
+		{ "CTE_450", "Wacom BambooFun 4x5",  2, 0x17 },
+		{ "CTE_650", "Wacom BambooFun 6x8",  3, 0x18 },
+		{ "CTE_631", "Wacom Bamboo1 Medium", 4, 0x19 },
 		{ NULL }
 	};
 
@@ -272,8 +274,8 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 	static USBSUBTYPE xCintiqV5[] =
 	{
 		{ "DTZ-21ux",  "Wacom Cintiq 21UX",  1, 0x3F },
-		{ "DTZ-20wsx", "Wacom Cintiq 20WSX", 2, 0x65 },
-		{ "DTZ-12wx",  "Wacom Cintiq 12WX",  3, 0x6C },
+		{ "DTZ-20wsx", "Wacom Cintiq 20WSX", 2, 0xC5 },
+		{ "DTZ-12wx",  "Wacom Cintiq 12WX",  3, 0xC6 },
 		{ NULL }
 	};
 
