@@ -32,10 +32,12 @@
 **   2007-06-05 0.1.3 - PC - Support Suppress and TwinView options
 **   2007-07-24 0.1.4 - PC - Support Screen_No option
 **   2008-01-17 0.1.5 - PC - Add DISPLAYTOGGLE command
+**   2008-03-24 0.1.6 - PC - Added Touch for serial Tablet PC (ISDv4)
+**   2008-08-27 0.1.7 - PC - Added get XORGVER to xsetwacom
 **
 ****************************************************************************/
 
-#define XSETWACOM_VERSION "0.1.5"
+#define XSETWACOM_VERSION "0.1.7"
 
 #include "../include/util-config.h"
 
@@ -344,10 +346,21 @@ static PARAMINFO gParamInfo[] =
 		XWACOM_PARAM_TPCBUTTON, VALUE_OPTIONAL, 
 		RANGE, 0, 1, BOOLEAN_VALUE, 1 },
 
+	{ "Touch",
+		"Turns on/off Touch events (default is enable/on). ",
+		XWACOM_PARAM_TOUCH, VALUE_OPTIONAL, 
+		RANGE, 0, 1, BOOLEAN_VALUE, 1 },
+
+	{ "Capacity",
+		"Touch sensitivity level (default is 3, "
+		"-1 for none capacitive tools).",
+		XWACOM_PARAM_CAPACITY, VALUE_OPTIONAL, 
+		RANGE, -1, 5, SINGLE_VALUE, 3 },
+
 	{ "CursorProx", 
 		"Sets cursor distance for proximity-out "
 		"in distance from the tablet.  "
-		"default is 10 for Intuos series, "
+		"(default is 10 for Intuos series, "
 		"42 for Graphire series).",
 		XWACOM_PARAM_CURSORPROX, VALUE_OPTIONAL, RANGE, 
 		0, 255, SINGLE_VALUE, 47 },
@@ -595,6 +608,10 @@ static PARAMINFO gParamInfo[] =
 		"Returns number of screens configured for the desktop. ",
 		XWACOM_PARAM_NUMSCREEN, VALUE_REQUIRED },
 
+	{ "XScaling", 
+		"Returns the status of XSCALING is set or not. ",
+		XWACOM_PARAM_XSCALING, VALUE_REQUIRED },
+
 	{ NULL }
 };
 
@@ -655,6 +672,7 @@ static int ListDev(WACOMCONFIG *hConfig, char** argv)
 		{ WACOMDEVICETYPE_CURSOR, "cursor" },
 		{ WACOMDEVICETYPE_STYLUS, "stylus" },
 		{ WACOMDEVICETYPE_ERASER, "eraser" },
+		{ WACOMDEVICETYPE_TOUCH, "touch" },
 		{ WACOMDEVICETYPE_PAD, "pad" }
 	};
 
@@ -706,11 +724,19 @@ static int ListParam(WACOMCONFIG *hConfig, char** argv)
 		"\tor specific keys and any other keys not listed as mod \n");
 	printf ("Examples:\n"
 		"  xsetwacom set stylus Button1 \"button 5\"\n"
+		"\t(emit mouse button 5 event)\n" 
 		"  xsetwacom set stylus Button3 \"dblclick 1\"\n"
+		"\t(emit mouse double left click event)\n" 
 		"  xsetwacom set pad Button2 \"core key ctrl alt F2\"\n"
+		"\t(emit ctrl + alt + F2 key event)\n" 
+		"  xsetwacom set pad Button3 \"core key quotedbl a space test space string quotedbl\"\n");
+	printf ("\t(emit keystroke \"a test string\" event)\n" 
 		"  xsetwacom set pad Button3 \"core key quotedbl a test string quotedbl\"\n"
+		"\t(emit keystroke \"ateststring\" event)\n" 
 		"  xsetwacom set pad striplup \"core key up\"\n"
-		"  xsetwacom set pad stripldn \"core key down\"\n");
+		"\t(emit up arrow key event)\n" 
+		"  xsetwacom set pad stripldn \"core key down\"\n"
+		"\t(emit down arrow key event)\n");
 
 	return 0;
 }
@@ -800,11 +826,15 @@ static int ParseValues(int nCount, const char* pszValues, int* nValues,
 		if ((p->nType == BOOLEAN_VALUE) &&
 			(!strcasecmp(pszValues,"on") ||
 			!strcasecmp(pszValues,"true") ||
+			!strcasecmp(pszValues,"enable") ||
+			!strcasecmp(pszValues,"1") ||
 			!strcasecmp(pszValues,"absolute")))
 				*nValue = 1;
 		else if ((p->nType == BOOLEAN_VALUE) &&
 			(!strcasecmp(pszValues,"off") ||
 			!strcasecmp(pszValues,"false") ||
+			!strcasecmp(pszValues,"disable") ||
+			!strcasecmp(pszValues,"0") ||
 			!strcasecmp(pszValues,"relative")))
 				*nValue = 0;
 		else if (p->nType == ACTION_VALUE)
