@@ -44,6 +44,7 @@ WacomDeviceClass* wcmDeviceClasses[] =
 
 extern int xf86WcmDevSwitchModeCall(LocalDevicePtr local, int mode);
 extern void xf86WcmChangeScreen(LocalDevicePtr local, int value);
+extern void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes);
 
 /*****************************************************************************
  * Static functions
@@ -76,7 +77,7 @@ void xf86WcmMappingFactor(LocalDevicePtr local)
 	if (!(priv->flags & ABSOLUTE_FLAG) || !priv->wcmMMonitor)
 	{
 		/* Get the current screen that the cursor is in */
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 		if (miPointerCurrentScreen())
 			priv->currentScreen = miPointerCurrentScreen()->myNum;
 #else
@@ -91,7 +92,7 @@ void xf86WcmMappingFactor(LocalDevicePtr local)
 		else if (priv->currentScreen == -1)
 		{
 			/* Get the current screen that the cursor is in */
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 			if (miPointerCurrentScreen())
 				priv->currentScreen = miPointerCurrentScreen()->myNum;
 #else
@@ -368,6 +369,7 @@ static void xf86WcmSendButtons(LocalDevicePtr local, int buttons, int rx, int ry
 	}
 }
 
+#ifdef WCM_KEY_SENDING_SUPPORT
 /*****************************************************************************
  * emitKeysym --
  *   Emit a keydown/keyup event
@@ -430,29 +432,25 @@ static void emitKeysym (DeviceIntPtr keydev, int keysym, int state)
 		{
 			for (j = ksr->minKeyCode; j <= ksr->maxKeyCode; j++)
 				if (ksr->map [(j - ksr->minKeyCode) * ksr->mapWidth] == XK_Shift_L)
-				break;
+					break;
 			if (state)
 				xf86PostKeyboardEvent (keydev, j, 1);
 			for (i = ksr->minKeyCode; i <= ksr->maxKeyCode; i++)
 				if (ksr->map [(i - ksr->minKeyCode) * ksr->mapWidth] == alt_keysym)
-				break;
+					break;
 			xf86PostKeyboardEvent (keydev, i, state);
-			if (state)
+			if (!state)
 				xf86PostKeyboardEvent (keydev, j, 0);
 		}
 		else
 			xf86Msg (X_WARNING, "Couldn't find key with code %08x on keyboard device %s\n",
-			 keysym, keydev->name);
+					keysym, keydev->name);
 		return;
 	}
 	xf86PostKeyboardEvent (keydev, i, state);
 }
 
-/*****************************************************************************
- * sendAButton --
- *   Send one button event, called by xf86WcmSendButtons
- ****************************************************************************/
-static int wcm_modifier [ ] = 
+static int wcm_modifier [ ] =
 {
 	XK_Shift_L,
 	XK_Control_L,
@@ -474,17 +472,50 @@ static int WcmIsModifier(int keysym)
 		}
 	return match;
 }
+#endif /*WCM_KEY_SENDING_SUPPORT*/
 
+static void sendKeystroke(LocalDevicePtr local, int button, unsigned *keyP, int kPress)
+{
+#ifndef WCM_KEY_SENDING_SUPPORT
+	ErrorF ("Error: [wacom] your X server doesn't support key events!\n");
+#else /* WCM_KEY_SENDING_SUPPORT */
+	if (button & AC_CORE)
+	{
+		int i = 0;
+
+		for (i=0; i<((button & AC_NUM_KEYS)>>20); i++)
+		{
+			/* modifier and key down then key up events */
+			if(kPress)
+			{
+				emitKeysym (local->dev, keyP[i], 1);
+				if (!WcmIsModifier(keyP[i]))
+					emitKeysym (local->dev, keyP[i], 0);
+			}
+			/* modifier up events */
+			else if (WcmIsModifier(keyP[i]))
+				emitKeysym (local->dev, keyP[i], 0);
+		}
+	}
+	else
+		ErrorF ("WARNING: [%s] without SendCoreEvents. Cannot emit key events!\n", local->name);
+#endif /* WCM_KEY_SENDING_SUPPORT */
+}
+
+/*****************************************************************************
+ * sendAButton --
+ *   Send one button event, called by xf86WcmSendButtons
+ ****************************************************************************/
 static void sendAButton(LocalDevicePtr local, int button, int mask,
 		int rx, int ry, int rz, int v3, int v4, int v5)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	WacomCommonPtr common = priv->common;
 	int is_absolute = priv->flags & ABSOLUTE_FLAG;
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 	int is_core = local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER);
 #endif
-	int i, button_idx, naxes = priv->naxes;
+	int button_idx, naxes = priv->naxes;
 
 	if (IsPad(priv))
 		button -= 8;
@@ -501,7 +532,7 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 	if (!button)  /* ignore this button event */
 		return;
 
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 	/* Switch the device to core mode, if required */
 	if (!is_core && (button & AC_CORE))
 		xf86XInputSetSendCoreEvents (local, TRUE);
@@ -521,39 +552,13 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 */		/* Dynamically modify the button map as required --
 		 * to be moved in the place where button mappings are changed
 		 */
-#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
-		/* +1 to be able to support Xorg xserver 1.4, which due to a
-		 * button translation bug will run the translation twice. By
-		 * adding 1 to the button# we're able to support at least a
-		 * simple button1 is button1 mapping. 
-		 */
-		button_idx++;
-#endif
 		local->dev->button->map [button_idx] = button & AC_CODE;
 		xf86PostButtonEvent(local->dev, is_absolute, button_idx,
 			mask != 0,0,naxes,rx,ry,rz,v3,v4,v5);
-
 		break;
 
 	case AC_KEY:
-		if (button & AC_CORE)
-		{
-			for (i=0; i<((button & AC_NUM_KEYS)>>20); i++)
-			{
-				/* button down to send modifier and key down then key up */
-				if (mask)
-				{
-					emitKeysym (inputInfo.keyboard, priv->keys[button_idx][i], 1);
-					if (!WcmIsModifier(priv->keys[button_idx][i]))
-						emitKeysym (inputInfo.keyboard, priv->keys[button_idx][i], 0);
-				}
-				/* button up to send modifier up */
-				else if (WcmIsModifier(priv->keys[button_idx][i]))
-					emitKeysym (inputInfo.keyboard, priv->keys[button_idx][i], 0);
-			}
-		}
-		else
-			ErrorF ("WARNING: Devices without SendCoreEvents cannot emit key events!\n");
+		sendKeystroke(local, button, priv->keys[button_idx], mask);
 		break;
 
 	case AC_MODETOGGLE:
@@ -598,12 +603,6 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		 * to be moved in the place where button mappings are changed.
 		 * Only left double is supported.
 		 */
-#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
-		/* Match the button sent to the actual pos due to a bug in Xorg
-		 * xserver 1.4. I.e. button1 is button1.
-		 */
-		button_idx = 1;
-#endif
 		local->dev->button->map [button_idx] = 1;
 
 		if (mask)
@@ -625,7 +624,7 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		break;
 	}
 
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 	/* Switch the device out of the core mode, if required */
 	if (!is_core && (button & AC_CORE))
 		xf86XInputSetSendCoreEvents (local, FALSE);
@@ -643,7 +642,7 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 	int fakeButton = 0, i, value = 0, naxes = priv->naxes;
 	unsigned  *keyP = 0;
 	int is_absolute = priv->flags & ABSOLUTE_FLAG;
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 	int is_core = local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER);
 #endif
 	DBG(10, priv->debugLevel, ErrorF("sendWheelStripEvents for %s \n", local->name));
@@ -740,7 +739,7 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 		"send fakeButton %x with value = %d \n", 
 		fakeButton, value));
 
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 	/* Switch the device to core mode, if required */
 	if (!is_core && (fakeButton & AC_CORE))
 		xf86XInputSetSendCoreEvents (local, TRUE);
@@ -749,48 +748,25 @@ static void sendWheelStripEvents(LocalDevicePtr local, const WacomDeviceState* d
 	{
 	    case AC_BUTTON:
 		/* send both button on/off in the same event for pad */
-		i = 0;
-#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
-		/* Match the button sent to the actual pos due to a bug in Xorg
-		 * xserver 1.4. I.e. button4 is button4. This makes it usable if
-		 * no exotic configuration is made.
-		 */
-		i = fakeButton & AC_CODE;
-#endif
-		local->dev->button->map [i] = fakeButton & AC_CODE;
+		local->dev->button->map [0] = fakeButton & AC_CODE;
 	
-		xf86PostButtonEvent(local->dev, is_absolute, i,
+		xf86PostButtonEvent(local->dev, is_absolute, 0,
 			1,0,naxes,x,y,z,v3,v4,v5);
 
-		xf86PostButtonEvent(local->dev, is_absolute, i,
+		xf86PostButtonEvent(local->dev, is_absolute, 0,
 			0,0,naxes,x,y,z,v3,v4,v5);
-
 	    break;
 
 	    case AC_KEY:
-		if (fakeButton & AC_CORE)
-		{
-			/* modifier and key down then key up events */
-			for (i=0; i<((fakeButton & AC_NUM_KEYS)>>20); i++)
-			{
-				emitKeysym (inputInfo.keyboard, keyP[i], 1);
-				if (!WcmIsModifier(keyP[i]))
-					emitKeysym (inputInfo.keyboard, keyP[i], 0);
-			}
-			/* modifier up events */
-			for (i=0; i<((fakeButton & AC_NUM_KEYS)>>20); i++)
-				if (WcmIsModifier(keyP[i]))
-					emitKeysym (inputInfo.keyboard, keyP[i], 0);
-		}
-		else
-			ErrorF ("WARNING: [%s] without SendCoreEvents. Cannot emit key events!\n", local->name);
+		sendKeystroke(local, fakeButton, keyP, 1);
+		sendKeystroke(local, fakeButton, keyP, 0);
 	    break;
 
 	    default:
 		ErrorF ("WARNING: [%s] unsupported event %x \n", local->name, fakeButton);
 	}
 
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 	/* Switch the device out of the core mode, if required
 	 */
 	if (!is_core && (fakeButton & AC_CORE))
@@ -807,12 +783,6 @@ static void sendCommonEvents(LocalDevicePtr local, const WacomDeviceState* ds, i
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	int buttons = ds->buttons;
-	int naxes = priv->naxes;
-
-	/* don't emit proximity events if device does not support proximity */
-	if ((local->dev->proximity && !priv->oldProximity))
-		xf86PostProximityEvent(local->dev, 1, 0, naxes,
-			x, y, z, v3, v4, v5);
 
 	if (priv->oldButtons != buttons)
 		xf86WcmSendButtons(local,buttons,x,y,z,v3,v4,v5);
@@ -874,7 +844,8 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		"b=%s b=%d tx=%d ty=%d wl=%d rot=%d th=%d\n",
 		(type == STYLUS_ID) ? "stylus" :
 			(type == CURSOR_ID) ? "cursor" : 
-			(type == ERASER_ID) ? "eraser" : "pad",
+			(type == ERASER_ID) ? "eraser" :
+			(type == TOUCH_ID) ? "touch" : "pad",
 		priv->oldProximity ? "true" : "false",
 		x, y, z, is_button ? "true" : "false", buttons,
 		tx, ty, wheel, rot, throttle));
@@ -884,18 +855,18 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 	{
 		tmp_coord = x;
 		x = y;
-		y = common->wcmMaxY - tmp_coord;
+		y = priv->wcmMaxY - tmp_coord;
 	}
 	else if (common->wcmRotate == ROTATE_CCW)
 	{
 		tmp_coord = y;
 		y = x;
-		x = common->wcmMaxX - tmp_coord;
+		x = priv->wcmMaxX - tmp_coord;
 	}
 	else if (common->wcmRotate == ROTATE_HALF)
 	{
-		x = common->wcmMaxX - x;
-		y = common->wcmMaxY - y;
+		x = priv->wcmMaxX - x;
+		y = priv->wcmMaxY - y;
 	}
 
 	if (IsCursor(priv)) 
@@ -931,6 +902,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		priv->oldZ = z;
 		priv->oldTiltX = tx;
 		priv->oldTiltY = ty;
+		priv->oldCapacity = ds->capacity;
 		priv->oldStripX = ds->stripx;
 		priv->oldStripY = ds->stripy;
 		priv->oldRot = rot;
@@ -994,8 +966,12 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			/* The +-0.4 is to increase the sensitivity in relative mode.
 			 * Must be sensitive to which way the tool is moved or one way
 			 * will get a severe penalty for small movements. */
-			x = (int)((double)(x - priv->topX) * priv->factorX + (x>=0?0.4:-0.4));
-			y = (int)((double)(y - priv->topY) * priv->factorY + (y>=0?0.4:-0.4));
+ 			if(is_absolute) {
+				x -= priv->topX;
+				y -= priv->topY;
+			}
+			x = (int)((double)x * priv->factorX + (x>=0?0.4:-0.4));
+			y = (int)((double)y * priv->factorY + (y>=0?0.4:-0.4));
 
 			/* map to a specific screen */
 			if (priv->screen_no != -1 || priv->twinview != TV_NONE)
@@ -1003,12 +979,19 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 				x += priv->screenTopX[priv->currentScreen];
 				y += priv->screenTopY[priv->currentScreen];
 			}
+			priv->currentSX = x;
+			priv->currentSY = y;
 #endif
-			sendCommonEvents(local, ds, x, y, z, v3, v4, v5);
+			/* don't emit proximity events if device does not support proximity */
+			if ((local->dev->proximity && !priv->oldProximity))
+				xf86PostProximityEvent(local->dev, 1, 0, naxes, x, y, z, v3, v4, v5);
 
+			/* Move the cursor to where it should be before sending button events */
 			if(!(priv->flags & BUTTONS_ONLY_FLAG))
 				xf86PostMotionEvent(local->dev, is_absolute,
 					0, naxes, x, y, z, v3, v4, v5);
+
+			sendCommonEvents(local, ds, x, y, z, v3, v4, v5);
 		}
 
 		/* not in proximity */
@@ -1016,10 +999,18 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		{
 			buttons = 0;
 
+#ifdef WCM_XORG_TABLET_SCALING
+			/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
+			 * for coordinate conversion at the moment */
+			x = priv->currentSX;
+			y = priv->currentSY;
+#endif
+
 			/* reports button up when the device has been
 			 * down and becomes out of proximity */
 			if (priv->oldButtons)
 				xf86WcmSendButtons(local,0,x,y,z,v3,v4,v5);
+
 			if (priv->oldProximity && local->dev->proximity)
 				xf86PostProximityEvent(local->dev,0,0,naxes,x,y,z,v3,v4,v5);
 		} /* not in proximity */
@@ -1032,6 +1023,11 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			y = 0;
 			if ( v3 || v4 || v5 )
 				xf86WcmSetScreen(local, &x, &y);
+
+			/* don't emit proximity events if device does not support proximity */
+			if ((local->dev->proximity && !priv->oldProximity))
+			xf86PostProximityEvent(local->dev, 1, 0, naxes, x, y, z, v3, v4, v5);
+
 			sendCommonEvents(local, ds, x, y, z, v3, v4, v5);
 			is_proximity = 1;
 			/* xf86PostMotionEvent is only needed to post the valuators
@@ -1064,6 +1060,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		priv->oldX = priv->currentX;
 		priv->oldY = priv->currentY;
 		priv->oldZ = z;
+		priv->oldCapacity = ds->capacity;
 		priv->oldTiltX = tx;
 		priv->oldTiltY = ty;
 		priv->oldStripX = ds->stripx;
@@ -1078,6 +1075,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		priv->oldX = 0;
 		priv->oldY = 0;
 		priv->oldZ = 0;
+		priv->oldCapacity = ds->capacity;
 		priv->oldTiltX = 0;
 		priv->oldTiltY = 0;
 		priv->oldStripX = 0;
@@ -1108,6 +1106,7 @@ static int xf86WcmSuppress(WacomCommonPtr common, const WacomDeviceState* dsOrig
 	if (ABS(dsOrig->tiltx - dsNew->tiltx) > suppress) returnV = 1;
 	if (ABS(dsOrig->tilty - dsNew->tilty) > suppress) returnV = 1;
 	if (ABS(dsOrig->pressure - dsNew->pressure) > suppress) returnV = 1;
+	if (ABS(dsOrig->capacity - dsNew->capacity) > suppress) returnV = 1;
 	if (ABS(dsOrig->throttle - dsNew->throttle) > suppress) returnV = 1;
 	if (ABS(dsOrig->rotation - dsNew->rotation) > suppress &&
 		(1800 - ABS(dsOrig->rotation - dsNew->rotation)) >  suppress) returnV = 1;
@@ -1234,7 +1233,8 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 
 #ifdef WCM_ENABLE_LINUXINPUT
 	/* Discard the first 2 USB packages due to events delay */
-	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && ds.device_type != PAD_ID )
+	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && 
+		ds.device_type != PAD_ID && ds.device_type != TOUCH_ID )
 	{
 		DBG(11, common->debugLevel, 
 			ErrorF("discarded %dth USB data.\n", 
@@ -1376,6 +1376,9 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 			case CURSOR_DEVICE_ID:
 				ds->device_type = CURSOR_ID;
 				break;
+			case TOUCH_DEVICE_ID:
+				ds->device_type = TOUCH_ID;
+				break;
 			default:
 				ds->device_type = idtotype(ds->device_id);
 		}
@@ -1394,6 +1397,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 	 * hand, the one with serial set to 0 if no match with the
 	 * specified serial exists) that is used for this event */
 	for (tool = common->wcmTool; tool; tool = tool->next)
+	{
 		if (tool->typeid == ds->device_type)
 		{
 			if (tool->serial == ds->serial_num)
@@ -1401,10 +1405,12 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 			else if (!tool->serial)
 				tooldef = tool;
 		}
+	}
 
 	/* Use default tool (serial == 0) if no specific was found */
 	if (!tool)
 		tool = tooldef;
+
 	/* 2: Find the associated area, and its InputDevice */
 	if (tool)
 	{
@@ -1447,6 +1453,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 				/* Send soft prox-out for the old area */
 				LocalDevicePtr oDev = outprox->device;
 				WacomDeviceState out = { 0 };
+				out.device_type = DEVICE_ID(((WacomDevicePtr)(oDev->private))->flags);
 				DBG(2, common->debugLevel, ErrorF("Soft prox-out for %s\n",
 					outprox->device->name));
 				xf86WcmSendEvents(oDev, &out);
@@ -1467,7 +1474,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 	/* Tool on the tablet when driver starts. This sometime causes
 	 * access errors to the device */
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+#if WCM_XINPUTABI_MAJOR == 0
 	if (!miPointerCurrentScreen())
 #else
 	if (pDev && !miPointerGetScreen(pDev->dev))
@@ -1490,7 +1497,7 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 		/* button 1 Threshold test */
 		int button = 1;
 		priv = pDev->private;
-		if ( IsStylus(priv) || IsEraser(priv))
+		if (IsStylus(priv) || IsEraser(priv))
 		{
 			if (filtered.pressure < common->wcmThreshold )
 				filtered.buttons &= ~button;
@@ -1500,6 +1507,13 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 			transPressureCurve(priv,&filtered);
 		}
 
+		/* touch capacity is supported */
+		if (IsTouch(priv) && common->wcmCapacityDefault >= 0)
+		{
+			if ((int)((filtered.capacity * 5) / common->wcmMaxZ) > common->wcmCapacity)
+				filtered.buttons |= button;
+		}
+
 		if (!(priv->flags & ABSOLUTE_FLAG) && !priv->hardProx)
 		{
 			/* initial current max distance */
@@ -1609,14 +1623,16 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 			if (priv->oldProximity)
 			{
-				if (abs(filtered.distance - common->wcmMaxCursorDist) > common->wcmCursorProxoutDist)
+				if (abs(filtered.distance - common->wcmMaxCursorDist) 
+						> common->wcmCursorProxoutDist)
 					filtered.proximity = 0;
 			}
 			/* once it is out. Don't let it in until a hard in */
 			/* or it gets inside wcmCursorProxoutDist */
 			else
 			{
-				if (abs(filtered.distance - common->wcmMaxCursorDist) > common->wcmCursorProxoutDist && ds->proximity)
+				if (abs(filtered.distance - common->wcmMaxCursorDist) > 
+						common->wcmCursorProxoutDist && ds->proximity)
 					return;
 				if (!ds->proximity)
 					return;	
@@ -1643,10 +1659,8 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
 
 int xf86WcmInitTablet(LocalDevicePtr local, const char* id, float version)
 {
-	WacomCommonPtr common = ((WacomDevicePtr)(local->private))->common;
-	WacomToolPtr toollist = common->wcmTool;
-	WacomToolAreaPtr arealist;
-	int temp;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
 	WacomModelPtr model = common->wcmModel;
 
 	/* Initialize the tablet */
@@ -1660,26 +1674,6 @@ int xf86WcmInitTablet(LocalDevicePtr local, const char* id, float version)
 	if (model->GetRanges && (model->GetRanges(local) != Success))
 		return !Success;
 	
-	/* Rotation rotates the Max Y and Y */
-	if (common->wcmRotate==ROTATE_CW || common->wcmRotate==ROTATE_CCW)
-	{
-		temp = common->wcmMaxX;
-		common->wcmMaxX = common->wcmMaxY;
-		common->wcmMaxY = temp;
-	}
-
-	for (; toollist; toollist=toollist->next)
-	{
-		arealist = toollist->arealist;
-		for (; arealist; arealist=arealist->next)
-		{
-			if (!arealist->bottomX) 
-				arealist->bottomX = common->wcmMaxX;
-			if (!arealist->bottomY)
-				arealist->bottomY = common->wcmMaxY;
-		}
-	}
-
 	/* Default threshold value if not set */
 	if (common->wcmThreshold <= 0)
 	{
@@ -1711,26 +1705,26 @@ int xf86WcmInitTablet(LocalDevicePtr local, const char* id, float version)
 	}
 
 	/* change the serial speed, if requested */
-	if (common->wcmLinkSpeed != 9600)
+	if (model->SetLinkSpeed)
 	{
-		if (model->SetLinkSpeed)
+		if (common->wcmLinkSpeed != 9600)
 		{
 			if (model->SetLinkSpeed(local) != Success)
 				return !Success;
 		}
-		else
-		{
-			ErrorF("Tablet does not support setting link "
-				"speed, or not yet implemented\n");
-		}
+	}
+	else
+	{
+		DBG(2, common->debugLevel, ErrorF("Tablet does not support setting link "
+			"speed, or not yet implemented\n"));
 	}
 
 	/* output tablet state as probed */
 	if (xf86Verbose)
-		ErrorF("%s Wacom %s tablet speed=%d maxX=%d maxY=%d maxZ=%d "
+		ErrorF("%s Wacom %s tablet speed=%d (%d) maxX=%d maxY=%d maxZ=%d "
 			"resX=%d resY=%d  tilt=%s\n",
 			XCONFIG_PROBED,
-			model->name, common->wcmLinkSpeed,
+			model->name, common->wcmLinkSpeed, common->wcmISDV4Speed, 
 			common->wcmMaxX, common->wcmMaxY, common->wcmMaxZ,
 			common->wcmResolX, common->wcmResolY,
 			HANDLE_TILT(common) ? "enabled" : "disabled");
@@ -1872,7 +1866,7 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 	priv->screenBottomY[0] = 0;
 	for (i=0; i<screenInfo.numScreens; i++)
 	{
-#ifdef WCM_XORG
+#ifdef WCM_HAVE_DIXSCREENORIGINS
 		priv->screenTopX[i] = dixScreenOrigins[i].x;
 		priv->screenTopY[i] = dixScreenOrigins[i].y;
 		priv->screenBottomX[i] = dixScreenOrigins[i].x;
@@ -1881,7 +1875,7 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 		DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialScreens from dix for \"%s\" "
 			"ScreenOrigins[%d].x=%d ScreenOrigins[%d].y=%d \n",
 			local->name, i, priv->screenTopX[i], i, priv->screenTopY[i]));
-#else
+#else /* WCM_HAVE_DIXSCREENORIGINS */
 		if (i > 0)
 		{
 			/* only support left to right in this case */
@@ -1890,7 +1884,7 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 			priv->screenBottomX[i] = priv->screenTopX[i];
 			priv->screenBottomY[i] = 0;
 		}
-#endif
+#endif /* WCM_HAVE_DIXSCREENORIGINS */
 		priv->screenBottomX[i] += screenInfo.screens[i]->width;
 		priv->screenBottomY[i] += screenInfo.screens[i]->height;
 
@@ -1900,3 +1894,148 @@ void xf86WcmInitialScreens(LocalDevicePtr local)
 			i, priv->screenBottomX[i], i, priv->screenBottomY[i]));
 	}
 }
+
+/*****************************************************************************
+ * rotateOneTool
+ ****************************************************************************/
+
+static void rotateOneTool(WacomDevicePtr priv)
+{
+	WacomCommonPtr common = priv->common;
+	WacomToolAreaPtr area = priv->toolarea;
+	int tmpTopX, tmpTopY, tmpBottomX, tmpBottomY, oldMaxX, oldMaxY;
+
+	DBG(10, priv->debugLevel, ErrorF("rotateOneTool for \"%s\" \n", priv->local->name));
+
+	oldMaxX = priv->wcmMaxX;
+	oldMaxY = priv->wcmMaxY;
+
+	tmpTopX = priv->topX;
+	tmpBottomX = priv->bottomX;
+	tmpTopY = priv->topY;
+	tmpBottomY = priv->bottomY;
+
+	if (common->wcmRotate == ROTATE_CW || common->wcmRotate == ROTATE_CCW)
+	{
+	    priv->wcmMaxX = oldMaxY;
+	    priv->wcmMaxY = oldMaxX;
+	}
+
+	switch (common->wcmRotate) {
+	      case ROTATE_CW:
+		area->topX = priv->topX = tmpTopY;
+		area->bottomX = priv->bottomX = tmpBottomY;
+		area->topY = priv->topY = oldMaxX - tmpBottomX;
+		area->bottomY = priv->bottomY =oldMaxX - tmpTopX;
+		break;
+	      case ROTATE_CCW:
+		area->topX = priv->topX = oldMaxY - tmpBottomY;
+		area->bottomX = priv->bottomX = oldMaxY - tmpTopY;
+		area->topY = priv->topY = tmpTopX;
+		area->bottomY = priv->bottomY = tmpBottomX;
+		break;
+	      case ROTATE_HALF:
+		area->topX = priv->topX = oldMaxX - tmpBottomX;
+		area->bottomX = priv->bottomX = oldMaxX - tmpTopX;
+		area->topY = priv->topY= oldMaxY - tmpBottomY;
+		area->bottomY = priv->bottomY = oldMaxY - tmpTopY;
+		break;
+	}
+	xf86WcmInitialScreens(priv->local);
+	xf86WcmMappingFactor(priv->local);
+	xf86WcmInitialCoordinates(priv->local, 0);
+	xf86WcmInitialCoordinates(priv->local, 1);
+
+	if (tmpTopX != priv->topX)
+		xf86ReplaceIntOption(priv->local->options, "TopX", priv->topX);
+	if (tmpTopY != priv->topY)
+		xf86ReplaceIntOption(priv->local->options, "TopY", priv->topY);
+	if (tmpBottomX != priv->bottomX)
+		xf86ReplaceIntOption(priv->local->options, "BottomX", priv->bottomX);
+	if (tmpBottomY != priv->bottomY)
+		xf86ReplaceIntOption(priv->local->options, "BottomY", priv->bottomY);
+}
+
+/*****************************************************************************
+ * xf86WcmRotateScreen
+ ****************************************************************************/
+
+void xf86WcmRotateScreen(LocalDevicePtr local, int value)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common = priv->common;
+	WacomDevicePtr tmppriv;
+	int oldRotation;
+	int tmpTopX, tmpTopY, tmpBottomX, tmpBottomY, oldMaxX, oldMaxY;
+
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmRotateScreen for \"%s\" \n", local->name));
+
+	if (common->wcmRotate == value) /* initialization */
+	{
+		rotateOneTool(priv);
+	}
+	else
+	{
+		oldRotation = common->wcmRotate;
+		common->wcmRotate = value;
+
+		/* rotate all devices at once! else they get misaligned */
+		for (tmppriv = common->wcmDevices; tmppriv; tmppriv = tmppriv->next)
+		{
+		    oldMaxX = tmppriv->wcmMaxX;
+		    oldMaxY = tmppriv->wcmMaxY;
+
+		    if (oldRotation == ROTATE_CW || oldRotation == ROTATE_CCW) 
+		    {
+			tmppriv->wcmMaxX = oldMaxY;
+			tmppriv->wcmMaxY = oldMaxX;
+		    }
+
+		    tmpTopX = tmppriv->topX;
+		    tmpBottomX = tmppriv->bottomX;
+		    tmpTopY = tmppriv->topY;
+		    tmpBottomY = tmppriv->bottomY;
+
+		    /* recover to the unrotated xy-rectangles */
+		    switch (oldRotation) {
+		      case ROTATE_CW:
+			tmppriv->topX = oldMaxY - tmpBottomY;
+			tmppriv->bottomX = oldMaxY - tmpTopY;
+			tmppriv->topY = tmpTopX;
+			tmppriv->bottomY = tmpBottomX;
+			break;
+		      case ROTATE_CCW:
+			tmppriv->topX = tmpTopY;
+			tmppriv->bottomX = tmpBottomY;
+			tmppriv->topY = oldMaxX - tmpBottomX;
+			tmppriv->bottomY = oldMaxX - tmpTopX;
+			break;
+		      case ROTATE_HALF:
+			tmppriv->topX = oldMaxX - tmpBottomX;
+			tmppriv->bottomX = oldMaxX - tmpTopX;
+			tmppriv->topY = oldMaxY - tmpBottomY;
+			tmppriv->bottomY = oldMaxY - tmpTopY;
+			break;
+		    }
+
+		    /* and rotate them to the new value */
+		    rotateOneTool(tmppriv);
+
+		    switch(value) {
+			case ROTATE_NONE:
+			    xf86ReplaceStrOption(local->options, "Rotate", "NONE");
+			break;
+			case ROTATE_CW:
+			    xf86ReplaceStrOption(local->options, "Rotate", "CW");
+			break;
+			case ROTATE_CCW:
+			    xf86ReplaceStrOption(local->options, "Rotate", "CCW");
+			break;
+			case ROTATE_HALF:
+			    xf86ReplaceStrOption(local->options, "Rotate", "HALF");
+			break;
+		    }
+		}
+	}
+
+}
