@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
+ * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,28 +66,15 @@ static Bool isdv4Detect(LocalDevicePtr local)
 
 static Bool isdv4Init(LocalDevicePtr local, char* id, float *version)
 {
-	int err;
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 
 	DBG(1, priv->debugLevel, ErrorF("initializing ISDV4 tablet\n"));
 
-	/* Try 19200 first */
+	/* Initial baudrate is 38400 */
 	if (xf86WcmSetSerialSpeed(local->fd, common->wcmISDV4Speed) < 0)
 		return !Success;
 
-	/* Send stop command to the tablet */
-	err = xf86WcmWrite(local->fd, WC_ISDV4_STOP, strlen(WC_ISDV4_STOP));
-	if (err == -1)
-	{
-		ErrorF("Wacom xf86WcmWrite error : %s\n", strerror(errno));
-		return !Success;
-	}
-
-	/* Wait 250 mSecs */
-	if (xf86WcmWait(250))
-		return !Success;
-
 	if(id)
 		strcpy(id, "ISDV4");
 	if(version)
@@ -100,100 +87,206 @@ static Bool isdv4Init(LocalDevicePtr local, char* id, float *version)
 }
 
 /*****************************************************************************
- * isdv4InitISDV4 -- Setup the device
+ * isdv4Query -- Query the device
  ****************************************************************************/
 
-static void isdv4InitISDV4(WacomCommonPtr common, const char* id, float version)
-{  
-	/* set parameters */
-	common->wcmProtocolLevel = 4;
-	common->wcmPktLength = 5;       /* length of a packet 
-					 * device packets are 9 bytes long,
-					 * multitouch are only 5 */
-	common->wcmResolX = 2540; 	/* tablet X resolution in points/inch */
-	common->wcmResolY = 2540; 	/* tablet Y resolution in points/inch */
-	common->wcmTPCButton = 1;	/* Tablet PC buttons on by default */
-	common->wcmTPCButtonDefault = 1;
-	common->tablet_id = 0x90;
-}
-static int isdv4GetRanges(LocalDevicePtr local)
+static int isdv4Query(LocalDevicePtr local, const char* query, char* data)
 {
-	char data[BUFFER_SIZE];
+	int err;
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
-	int maxtry = MAXTRY, nr;
 	WacomCommonPtr common =	priv->common;
 
-	DBG(2, priv->debugLevel, ErrorF("getting ISDV4 Ranges\n"));
-	/* Send query command to the tablet */
-	do
-	{
-		nr = xf86WcmWrite(local->fd, WC_ISDV4_QUERY, strlen(WC_ISDV4_QUERY));
-		if ((nr == -1) && (errno != EAGAIN))
-		{
-			ErrorF("Wacom xf86WcmWrite error : %s", strerror(errno));
-			return !Success;
-		}
-		maxtry--;
-	} while ((nr == -1) && maxtry);
+	DBG(1, priv->debugLevel, ErrorF("Querying ISDV4 tablet\n"));
 
-	if (maxtry == 0)
+	/* Send stop command to the tablet */
+	err = xf86WcmWrite(local->fd, WC_ISDV4_STOP, strlen(WC_ISDV4_STOP));
+	if (err == -1)
 	{
-		ErrorF("Wacom unable to xf86WcmWrite request query command "
-				"after %d tries\n", MAXTRY);
+		ErrorF("Wacom xf86WcmWrite ISDV4_STOP error : %s\n", strerror(errno));
+		return !Success;
+	}
+
+	/* Wait 250 mSecs */
+	if (xf86WcmWait(250))
+		return !Success;
+		
+	/* Send query command to the tablet */
+	if (!xf86WcmWriteWait(local->fd, query))
+	{
+		ErrorF("Wacom unable to xf86WcmWrite request %s ISDV4 query command "
+			"after %d tries\n", query, MAXTRY);
 		return !Success;
 	}
 
 	/* Read the control data */
-	maxtry = MAXTRY;
-	do
+	if (!xf86WcmWaitForTablet(local->fd, data, 11))
 	{
-		if ((nr = xf86WcmWaitForTablet(local->fd)) > 0)
+		/* Try 19200 if it is not a touch query */
+		if (common->wcmISDV4Speed != 19200 && strcmp(query, WC_ISDV4_TOUCH_QUERY))
 		{
-			nr = xf86WcmRead(local->fd, data, 11);
-			if ((nr == -1) && (errno != EAGAIN))
-			{
-				ErrorF("Wacom xf86WcmRead error : %s\n", strerror(errno));
+			common->wcmISDV4Speed = 19200;
+			if (xf86WcmSetSerialSpeed(local->fd, common->wcmISDV4Speed) < 0)
 				return !Success;
-			}
+ 			return isdv4Query(local, query, data);
+		}
+		else
+		{
+			ErrorF("Wacom unable to read ISDV4 %s data "
+				"after %d tries at (%d)\n", query, MAXTRY, common->wcmISDV4Speed);
+			return !Success;
 		}
-		maxtry--;  
-	} while ( nr <= 0 && maxtry );
-
-	if (maxtry == 0 && nr <= 0 )
-	{
-		ErrorF("Wacom unable to read ISDV4 control data "
-				"after %d tries\n", MAXTRY);
-		return !Success;
 	}
 
 	/* Control data bit check */
 	if ( !(data[0] & 0x40) )
 	{
-		/* Try 38400 now */
-		if (common->wcmISDV4Speed != 38400)
+		/* Try 19200 if it is not a touch query */
+		if (common->wcmISDV4Speed != 19200 && strcmp(query, WC_ISDV4_TOUCH_QUERY))
 		{
-			common->wcmISDV4Speed = 38400;
-			if(isdv4Init(local, NULL, NULL) != Success)
+			common->wcmISDV4Speed = 19200;
+			if (xf86WcmSetSerialSpeed(local->fd, common->wcmISDV4Speed) < 0)
 				return !Success;
-			return isdv4GetRanges(local);
+ 			return isdv4Query(local, query, data);
 		}
 		else
 		{
-			ErrorF("Wacom Query ISDV4 error magic error \n");
+			ErrorF("Wacom ISDV4 control data (%x) error in %s query\n", data[0], query);
 			return !Success;
 		}
 	}
-	
-	common->wcmMaxZ = ( data[5] | ((data[6] & 0x07) << 7) );
-	common->wcmMaxX = ( (data[1] << 9) | (data[2] << 2) 
-				| ( (data[6] & 0x60) >> 5) );      
-	common->wcmMaxY = ( (data[3] << 9) | (data[4] << 2 ) 
-				| ( (data[6] & 0x18) >> 3) );
-	common->wcmVersion = ( data[10] | (data[9] << 7) );
 
 	return Success;
 }
 
+/*****************************************************************************
+ * isdv4InitISDV4 -- Setup the device
+ ****************************************************************************/
+
+static void isdv4InitISDV4(WacomCommonPtr common, const char* id, float version)
+{
+	/* set parameters */
+	common->wcmProtocolLevel = 4;
+	common->wcmPktLength = 9;       /* length of a packet 
+					 * device packets are 9 bytes
+					 * resistive touch is 5 bytes
+					 * capacitive touch is 7 bytes 
+					 */
+
+	/* digitizer X resolution in points/inch */
+	common->wcmResolX = 2540; 	
+	/* digitizer Y resolution in points/inch */
+	common->wcmResolY = 2540; 	
+
+	/* no touch */
+	common->tablet_id = 0x90;
+
+	/* tilt disabled */
+	common->wcmFlags &= ~TILT_ENABLED_FLAG;
+}
+
+
+/*****************************************************************************
+ * isdv4GetRanges -- get ranges of the device
+ ****************************************************************************/
+
+static int isdv4GetRanges(LocalDevicePtr local)
+{
+	char data[BUFFER_SIZE];
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common =	priv->common;
+
+	DBG(2, priv->debugLevel, ErrorF("getting ISDV4 Ranges\n"));
+
+	/* Send query command to the tablet */
+	if (isdv4Query(local, WC_ISDV4_QUERY, data) == Success)
+	{
+		/* transducer data */
+		common->wcmMaxZ = ( data[5] | 
+			((data[6] & 0x07) << 7) );
+		common->wcmMaxX = ( (data[1] << 9) | 
+			(data[2] << 2) | ( (data[6] & 0x60) >> 5) );      
+		common->wcmMaxY = ( (data[3] << 9) | (data[4] << 2 ) 
+			| ( (data[6] & 0x18) >> 3) );
+		if (data[7] && data[8])
+		{
+			common->wcmMaxtiltX = data[7] + 1;
+			common->wcmMaxtiltY = data[8] + 1;
+			common->wcmFlags |= TILT_ENABLED_FLAG;
+		}
+			
+		common->wcmVersion = ( data[10] | (data[9] << 7) );
+	}
+	else
+		return !Success;
+
+	if (common->wcmISDV4Speed != 19200)
+	{
+		/* Touch might be supported. Send a touch query command */
+		if (isdv4Query(local, WC_ISDV4_TOUCH_QUERY, data) == Success)
+		{
+			/* default to 0x93 (resisitive touch) */
+			common->wcmPktLength = 5;
+			common->tablet_id = 0x93;
+
+			if (data[0] & 0x41)
+			{
+				/* tablet model */
+				switch (data[2] & 0x07)
+				{
+					case 0x01:
+						common->wcmPktLength = 7;
+						common->tablet_id = 0x9A;
+					break;
+					case 0x02:
+					case 0x04:
+						common->wcmPktLength = 7;
+						common->tablet_id = 0x9F;
+					break;
+				}
+
+				/* touch logical size */
+				common->wcmMaxTouchX = common->wcmMaxTouchY = (int)data[1];
+
+				/* Max capacity */
+				common->wcmMaxCapacity = (int)data[7];
+
+				if (common->wcmMaxCapacity)
+					common->wcmCapacityDefault = 3;
+				else
+				{
+					common->wcmCapacityDefault = -1;
+					common->wcmCapacity = -1;
+				}
+			}
+		}
+
+		if (common->wcmMaxX && common->wcmMaxY && common->wcmMaxTouchX)
+		{
+			char *s = xf86FindOptionValue(local->options, "Touch");
+			if ( !s || (strstr(s, "on")) )  /* touch option is on */
+			{
+				common->wcmTouch = 1;
+			}
+
+			/* TouchDefault was off for all devices */
+			/* defaults to enable when touch is supported */
+			common->wcmTouchDefault = 1;
+
+			/* Touch resolution */
+			common->wcmTouchResolX = common->wcmMaxTouchX * 
+				common->wcmResolX / common->wcmMaxX;
+			common->wcmTouchResolY = common->wcmMaxTouchY * 
+				common->wcmResolY / common->wcmMaxY;
+		}
+	}
+
+	DBG(2, priv->debugLevel, ErrorF("isdv4GetRanges speed=%d maxX=%d maxY=%d "
+		"maxZ=%d resX=%d resY=%d \n", common->wcmISDV4Speed, 
+		common->wcmMaxX, common->wcmMaxY, common->wcmMaxZ,
+		common->wcmResolX, common->wcmResolY));
+	return Success;
+}
+
 static int isdv4StartTablet(LocalDevicePtr local)
 {
 	int err;
@@ -210,31 +303,95 @@ static int isdv4StartTablet(LocalDevicePtr local)
 	return Success;
 }
 
+void setPktLenght(LocalDevicePtr local)
+{
+	WacomCommonPtr common = ((WacomDevicePtr)local->private)->common;
+
+	if (common->wcmMaxCapacity)
+	common->wcmPktLength = 7;
+	else
+	common->wcmPktLength = 5;
+	return;
+}
+
 static int isdv4Parse(LocalDevicePtr local, const unsigned char* data)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 	WacomDeviceState* last = &common->wcmChannel[0].valid.state;
 	WacomDeviceState* ds;
-	int n, cur_type, ismt = 0;
-	static int lastismt = 0;
+	int n, cur_type, channel = 0;
+	static int touchInProx;
 
 	DBG(10, common->debugLevel, ErrorF("isdv4Parse \n"));
 
-	/* determine the type of message */
-	if (data[0] & 0x10)
+	/* determine the type of message (touch or stylus)*/
+	if (data[0] & 0x18) /* not a pen */
 	{
-		ismt = 1;
-		common->wcmPktLength = 5;
+		if ((common->wcmPktLength == 9 && last->proximity ) || 
+				!common->wcmTouch )
+		{
+			if ((data[0] & 0x10) && (!(data[0] & 0x01))) /* a touch out-prox data */
+				touchInProx = 0;
+			else
+				touchInProx = 1;
+
+			/* ignore touch event */
+			setPktLenght(local);
+			return common->wcmPktLength;
+		}
+		else
+		{
+			if (data[0] & 0x10) /* a touch data */
+			{
+				if (!touchInProx)
+				{
+					setPktLenght(local);
+					channel = 1;
+				} 
+				else if (!(data[0] & 0x01)) /* touch out-prox */
+				{
+					touchInProx = 0;
+					setPktLenght(local);
+					channel = 1;
+				} 
+				else
+				{
+					setPktLenght(local);
+					/* ignore touch event */
+					return common->wcmPktLength;
+				}
+			}
+			else
+			{
+				setPktLenght(local);
+				/* ignore touch event */
+				return common->wcmPktLength;
+			}
+		}
 	}
 	else
 	{
-		common->wcmPktLength = 9;
-		if (common->buffer + common->bufpos - data < common->wcmPktLength)
+		/* touch was in control */
+		if (common->wcmChannel[1].valid.state.proximity)
 		{
-			/* we can't handle this yet */
+			/* let touch go */
+			WacomDeviceState out = { 0 };
+			out.device_type = TOUCH_ID;
+			xf86WcmEvent(common, 1, &out);
 			return 0;
 		}
+		else
+		{
+			common->wcmPktLength = 9;
+			channel = 0;
+		}
+	}
+
+	if (common->buffer + common->bufpos - data < common->wcmPktLength)
+	{
+		/* we can't handle this yet */
+		return common->wcmPktLength;
 	}
 
 	if ((n = xf86WcmSerialValidate(common,data)) > 0)
@@ -242,34 +399,24 @@ static int isdv4Parse(LocalDevicePtr local, const unsigned char* data)
 	else
 	{
 		/* Coordinate data bit check */
-		if (data[0] & 0x40)
+		if (data[0] & 0x40) /* control data */
 			return common->wcmPktLength;
 	}
+
 	/* pick up where we left off, minus relative values */
-	ds = &common->wcmChannel[0].work;
+	ds = &common->wcmChannel[channel].work;
 	RESET_RELATIVE(*ds);
 
-	if (ismt)
+	if (common->wcmPktLength == 5 ) /* a touch */
 	{
-		if (!lastismt && last->pressure)
-		{
-			/* pen sends both pen and MultiTouch input, 
-			 * since pressing it creates pressure. 
-			 * We only want the pen input though.
-			 */
-			return common->wcmPktLength;
-		}
-		lastismt = ismt;
-
-		/* MultiTouch input is comparably simple */
-		ds->proximity = 0;
-		ds->x = (((((int)data[1]) << 7) | ((int)data[2])) - 18) * common->wcmMaxX / 926;
-		ds->y = (((((int)data[3]) << 7) | ((int)data[4])) - 51) * common->wcmMaxY / 934;
-		ds->pressure = (data[0] & 0x01) * common->wcmMaxZ;
-		ds->buttons = 1;
-		ds->device_id = STYLUS_DEVICE_ID;
-		ds->device_type = 0;
-		DBG(8, priv->debugLevel, ErrorF("isdv4Parse MultiTouch\n"));
+		/* MultiTouch input only has 5 bytes of data */
+		ds->x = (((int)data[1]) << 7) | ((int)data[2]);
+		ds->y = (((int)data[3]) << 7) | ((int)data[4]);
+		ds->buttons = ds->proximity = data[0] & 0x01;
+		ds->device_type = TOUCH_ID;
+		ds->device_id = TOUCH_DEVICE_ID;
+		DBG(8, priv->debugLevel, ErrorF("isdv4Parse MultiTouch "
+			"%s proximity \n", ds->proximity ? "in" : "out of"));
 	}
 	else
 	{
@@ -294,9 +441,9 @@ static int isdv4Parse(LocalDevicePtr local, const unsigned char* data)
 		if (!last->proximity && ds->proximity) 
 			ds->device_type = cur_type;
 		/* check on previous proximity */
-		else if (cur_type == STYLUS_ID && ds->proximity)
+		else if (ds->buttons && ds->proximity)
 		{
-			/* we were fooled by tip and second
+			/* we might have been fooled by tip and second
 			 * sideswitch when it came into prox */
 			if ((ds->device_type != cur_type) &&
 				(ds->device_type == ERASER_ID))
@@ -308,8 +455,8 @@ static int isdv4Parse(LocalDevicePtr local, const unsigned char* data)
 			}
 		}
 
-		ds->device_id = (ds->device_type == CURSOR_ID) ? 
-			CURSOR_DEVICE_ID : STYLUS_DEVICE_ID;
+		ds->device_id = (ds->device_type == ERASER_ID) ? 
+			ERASER_DEVICE_ID : STYLUS_DEVICE_ID;
 
 		/* don't send button 3 event for eraser 
 		 * button 1 event will be sent by testing presure level
@@ -324,7 +471,7 @@ static int isdv4Parse(LocalDevicePtr local, const unsigned char* data)
 			ds->device_type == ERASER_ID ? "ERASER " :
 			ds->device_type == STYLUS_ID ? "STYLUS" : "NONE"));
 	}
-	xf86WcmEvent(common,0,ds);
+	xf86WcmEvent(common, channel, ds);
 	return common->wcmPktLength;
 }
 
