@@ -240,6 +240,60 @@ static void serialParseP4Common(LocalDevicePtr local, const unsigned char* data,
 	bit 0  Yt1
 	*/
 
+/*****************************************************************************
+ * xf86WcmWrite --
+ *   send a request 
+ ****************************************************************************/
+
+int xf86WcmWriteWait(int fd, const char* request)
+{
+	int len, maxtry = MAXTRY;
+
+	/* send request string */
+	do
+	{
+		len = xf86WriteSerial(fd, request, strlen(request));
+		if ((len == -1) && (errno != EAGAIN))
+		{
+			ErrorF("Wacom xf86WcmWriteWait error : %s", strerror(errno));
+			return 0;
+		}
+
+		maxtry--;
+
+	} while ((len <= 0) && maxtry);
+
+	return maxtry;
+}
+
+/*****************************************************************************
+ * xf86WcmWaitForTablet --
+ *   wait for tablet data 
+ ****************************************************************************/
+
+int xf86WcmWaitForTablet(int fd, char* answer, int size)
+{
+	int len, maxtry = MAXTRY;
+
+	/* Read size bytes of the answer */
+	do
+	{
+		if ((len = xf86WaitForInput(fd, 1000000)) > 0)
+		{
+			len = xf86WcmRead(fd, answer, size);
+			if ((len == -1) && (errno != EAGAIN))
+			{
+				ErrorF("Wacom xf86WcmRead error : %s\n",
+						strerror(errno));
+				return 0;
+			}
+		}
+		maxtry--;
+	} while ((len <= 0) && maxtry);
+
+	return maxtry;
+}
+
 /*****************************************************************************
  * xf86WcmSendRequest --
  *   send a request and wait for the answer.
@@ -249,25 +303,14 @@ static void serialParseP4Common(LocalDevicePtr local, const unsigned char* data,
 
 char* xf86WcmSendRequest(int fd, const char* request, char* answer, int maxlen)
 {
-	int len, nr;
+	int len, nr = 0;
 	int maxtry = MAXTRY;
 
 	if (maxlen < 3)
 		return NULL;
   
-	/* send request string */
-	do
-	{
-		len = xf86WcmWrite(fd, request, strlen(request));
-		if ((len == -1) && (errno != EAGAIN))
-		{
-			ErrorF("Wacom xf86WcmWrite error : %s", strerror(errno));
-			return NULL;
-		}
-		maxtry--;
-	} while ((len == -1) && maxtry);
-
-	if (maxtry == 0)
+	/* wait for request return */
+	if (!xf86WcmWriteWait(fd, request))
 	{
 		ErrorF("Wacom unable to xf86WcmWrite request string '%s' "
 				"after %d tries\n", request, MAXTRY);
@@ -276,92 +319,61 @@ char* xf86WcmSendRequest(int fd, const char* request, char* answer, int maxlen)
 
 	do
 	{
-		maxtry = MAXTRY;
-
 		/* Read the first byte of the answer which must
 		 * be equal to the first byte of the request.
 		 */
-		do
-		{
-			if ((nr = xf86WcmWaitForTablet(fd)) > 0)
-			{
-				nr = xf86WcmRead(fd, answer, 1);
-				if ((nr == -1) && (errno != EAGAIN))
-				{
-					ErrorF("Wacom xf86WcmRead error : %s\n",
-							strerror(errno));
-					return NULL;
-				}
-			}
-			maxtry--;
-		} while ((answer[0] != request[0]) && maxtry);
-
-		if (maxtry == 0)
+		maxtry = xf86WcmWaitForTablet(fd, answer, 1);
+		if (answer[0] != request[0])
 		{
 			ErrorF("Wacom unable to read first byte of "
 					"request '%c%c' answer after %d tries\n",
-					request[0], request[1], MAXTRY);
+					request[0], request[1], maxtry);
 			return NULL;
 		}
 
 		/* Read the second byte of the answer which must be equal
 		 * to the second byte of the request. */
-		do
+		if (!xf86WcmWaitForTablet(fd, answer+1, 1))
 		{
-			maxtry = MAXTRY;
-			do
-			{
-				if ((nr = xf86WcmWaitForTablet(fd)) > 0)
-				{
-					nr = xf86WcmRead(fd, answer+1, 1);
-					if ((nr == -1) && (errno != EAGAIN))
-					{
-						ErrorF("Wacom xf86WcmRead error : %s\n",
-								strerror(errno));
-						return NULL;
-					}
-				}
-				maxtry--;
-			} while ((nr <= 0) && maxtry);
+			ErrorF("Wacom unable to read second byte of "
+				"request '%c%c' answer after %d "
+				"tries\n", request[0], request[1], maxtry);
+			return NULL;
+		}
 
-			if (maxtry == 0)
-			{
-				ErrorF("Wacom unable to read second byte of "
-					"request '%c%c' answer after %d "
-					"tries\n", request[0], request[1],
-					MAXTRY);
-				return NULL;
-			}
+		if (answer[1] != request[1])
+			answer[0] = answer[1];
 
-			if (answer[1] != request[1])
-				answer[0] = answer[1];
+	} while ((answer[0] == request[0]) && (answer[1] != request[1]));
 
-		} while ((answer[0] == request[0]) && (answer[1] != request[1]));
-
-	} while ((answer[0] != request[0]) && (answer[1] != request[1]));
 
 	/* Read until we don't get anything or timeout. */
 
 	len = 2;
-	maxtry = MAXTRY;
 	do
 	{
-		do
+		if (len == 2)
 		{
-			if ((nr = xf86WcmWaitForTablet(fd)) > 0)
+			if (!xf86WcmWaitForTablet(fd, answer+len, 1))
 			{
-				nr = xf86WcmRead(fd, answer+len, 1);
-				if ((nr == -1) && (errno != EAGAIN))
-				{
-					ErrorF("Wacom xf86WcmRead error : %s\n", strerror(errno));
-					return NULL;
-				}
+				ErrorF("Wacom unable to read last byte of request '%c%c' "
+					"answer after %d tries\n",
+					request[0], request[1], MAXTRY);
+				return NULL;
 			}
-			else
+			len++;
+		}
+
+		if ((nr = xf86WaitForInput(fd, 1000000)) > 0)
+		{
+			nr = xf86WcmRead(fd, answer+len, 1);
+			if ((nr == -1) && (errno != EAGAIN))
 			{
-				if (len == 2) maxtry--;
+				ErrorF("Wacom xf86WcmRead in xf86WcmSendRequest error : %s\n",
+						strerror(errno));
+				return NULL;
 			}
-		} while ((nr <= 0) && len == 2 && maxtry);
+		}
 
 		if (nr > 0)
 		{
@@ -370,12 +382,6 @@ char* xf86WcmSendRequest(int fd, const char* request, char* answer, int maxlen)
 				return NULL;
 		}
 
-		if (maxtry == 0)
-		{
-			ErrorF("Wacom unable to read last byte of request '%c%c' answer after %d tries\n",
-					request[0], request[1], MAXTRY);
-			break;
-		}
 	} while (nr > 0);
 
 	if (len <= 3)
@@ -944,6 +950,9 @@ static void serialInitCintiq(WacomCommonPtr common, const char* id, float versio
 		/* PL-800  */
 		common->wcmMaxZ = 511;
 	}
+
+	/* tilt disabled */
+	common->wcmFlags &= ~TILT_ENABLED_FLAG;
 }
 
 static void serialInitPenPartner(WacomCommonPtr common, const char* id, float version)
@@ -955,6 +964,9 @@ static void serialInitPenPartner(WacomCommonPtr common, const char* id, float ve
 	common->wcmMaxZ = 255;
 	common->wcmResolX = 1000; /* tablet X resolution in points/inch */
 	common->wcmResolY = 1000; /* tablet Y resolution in points/inch */
+
+	/* tilt disabled */
+	common->wcmFlags &= ~TILT_ENABLED_FLAG;
 }
 
 static void serialInitGraphire(WacomCommonPtr common, const char* id, float version)
@@ -969,6 +981,9 @@ static void serialInitGraphire(WacomCommonPtr common, const char* id, float vers
 	common->wcmMaxZ = 511;
 	common->wcmResolX = 1016; /* tablet X resolution in points/inch */
 	common->wcmResolY = 1016; /* tablet Y resolution in points/inch */
+
+	/* tilt disabled */
+	common->wcmFlags &= ~TILT_ENABLED_FLAG;
 }
 
 static void serialInitProtocol4(WacomCommonPtr common, const char* id, float version)
@@ -986,6 +1001,9 @@ static void serialInitProtocol4(WacomCommonPtr common, const char* id, float ver
 		else
 			common->wcmMaxZ = 120;
 	}
+
+	/* tilt disabled */
+	common->wcmFlags &= ~TILT_ENABLED_FLAG;
 }
 
 static void serialGetResolution(LocalDevicePtr local)
@@ -1234,24 +1252,25 @@ static void serialParseP4Common(LocalDevicePtr local,
 		ds->tiltx = (data[7] & TILT_BITS);
 		ds->tilty = (data[8] & TILT_BITS);
 		if (data[7] & TILT_SIGN_BIT)
-			ds->tiltx -= 64;
+			ds->tiltx -= common->wcmMaxtiltX/2;
 		if (data[8] & TILT_SIGN_BIT)
-			ds->tilty -= 64;
+			ds->tilty -= common->wcmMaxtiltY/2;
 	}
 
 	/* first time into prox */
 	if (!last->proximity && ds->proximity)
 		ds->device_type = cur_type;
 	/* check on previous proximity */
-	else if (is_stylus && ds->proximity)
+	else if (ds->buttons && ds->proximity)
 	{
-		/* we were fooled by tip and second
+		/* we might have been fooled by tip and second
 		 * sideswitch when it came into prox */
 		if ((ds->device_type != cur_type) &&
 			(ds->device_type == ERASER_ID))
 		{
 			/* send a prox-out for old device */
 			WacomDeviceState out = { 0 };
+			out.device_type = ERASER_ID;
 			xf86WcmEvent(common, 0, &out);
 			ds->device_type = cur_type;
 		}
@@ -1289,8 +1308,9 @@ int xf86WcmSerialValidate(WacomCommonPtr common, const unsigned char* data)
 				((i!=0) && (data[i] & HEADER_BIT)) )
 		{
 			bad = 1;
-			ErrorF("xf86WcmSerialValidate: bad magic at %d "
-				"v=%x l=%d\n", i, data[i], common->wcmPktLength);
+			if (i!=1)
+				ErrorF("xf86WcmSerialValidate: bad magic at %d "
+					"v=%x l=%d\n", i, data[i], common->wcmPktLength);
 			if (i!=0 && (data[i] & HEADER_BIT)) return i;
 		}
 	}
