@@ -338,6 +338,22 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial);
 		usbDetectConfig,      /* detect hardware buttons etc */
 	};
 
+	static WacomModel usbTabletPC =
+	{
+		"USB TabletPC",
+		usbInitProtocol4,
+		NULL,                 /* resolution not queried */
+		usbWcmGetRanges,
+		NULL,                 /* reset not supported */
+		NULL,                 /* tilt automatically enabled */
+		NULL,                 /* suppress implemented in software */
+		NULL,                 /* link speed unsupported */
+		NULL,                 /* start not supported */
+		usbParse,
+		NULL,                 /* input filtering */
+		usbDetectConfig,      /* detect hardware buttons etc */
+	};
+
 /*****************************************************************************
  * usbDetect --
  *   Test if the attached device is USB.
@@ -411,6 +427,7 @@ static struct
 	{ 0x16, 2032, 2032, &usbGraphire4  }, /* Graphire4 6x8 */ 
 	{ 0x17, 2540, 2540, &usbBambooFun  }, /* BambooFun 4x5 */
 	{ 0x18, 2540, 2540, &usbBambooFun  }, /* BambooFun 6x8 */
+	{ 0x19, 2032, 2032, &usbBamboo1    }, /* Bamboo1 Medium*/ 
 	{ 0x81, 2032, 2032, &usbGraphire4  }, /* Graphire4 6x8 BlueTooth */
 
 	{ 0x20, 2540, 2540, &usbIntuos     }, /* Intuos 4x5 */
@@ -432,6 +449,7 @@ static struct
 	{ 0x39,  508,  508, &usbCintiq     }, /* PL710 */ 
 	{ 0xC0,  508,  508, &usbCintiq     }, /* DTF720 */
 	{ 0xC4,  508,  508, &usbCintiq     }, /* DTF521 */ 
+	{ 0xC7, 2540, 2540, &usbCintiq     }, /* DTU1931 */
 
 	{ 0x41, 2540, 2540, &usbIntuos2    }, /* Intuos2 4x5 */
 	{ 0x42, 2540, 2540, &usbIntuos2    }, /* Intuos2 6x8 */
@@ -460,7 +478,11 @@ static struct
 
 	{ 0x3F, 5080, 5080, &usbCintiqV5   }, /* Cintiq 21UX */ 
 	{ 0xC5, 5080, 5080, &usbCintiqV5   }, /* Cintiq 20WSX */ 
-	{ 0xC6, 5080, 5080, &usbCintiqV5   }  /* Cintiq 12WX */ 
+	{ 0xC6, 5080, 5080, &usbCintiqV5   }, /* Cintiq 12WX */ 
+
+	{ 0x90, 2540, 2540, &usbTabletPC   }, /* TabletPC 0x90 */ 
+	{ 0x93, 2540, 2540, &usbTabletPC   }, /* TabletPC 0x93 */
+	{ 0x9A, 2540, 2540, &usbTabletPC   }  /* TabletPC 0x9A */
 };
 
 Bool usbWcmInit(LocalDevicePtr local, char* id, float *version)
@@ -490,6 +512,30 @@ Bool usbWcmInit(LocalDevicePtr local, char* id, float *version)
 				common->wcmResolX = WacomModelDesc [i].xRes;
 				common->wcmResolY = WacomModelDesc [i].yRes;
 			}
+		/* initialize capacity parameters for touch */
+		if (common->tablet_id == 0x9A)
+		{
+			common->wcmCapacity = 0;
+			common->wcmCapacityDefault = 3; 
+		}
+		else
+		{
+			common->wcmCapacity = -1;
+			common->wcmCapacityDefault = -1; 
+		}
+
+		if (common->tablet_id == 0x9A || common->tablet_id == 0x93)
+		{
+			char *s = xf86FindOptionValue(local->options, "Touch");
+			if ( !s || (strstr(s, "on")) )  /* touch option is on */
+			{
+				common->wcmTouch = 1;
+			}
+
+			/* TouchDefault was off for all devices */
+			/* defaults to enable when touch is supported */
+			common->wcmTouchDefault = 1;
+		}
 	}
 
 	if (!common->wcmModel)
@@ -525,8 +571,6 @@ Bool usbWcmInit(LocalDevicePtr local, char* id, float *version)
 	else
 		common->nbuttons = 5;
 
-	common->wcmFlags |= TILT_ENABLED_FLAG;
-
 	return Success;
 }
 
@@ -537,6 +581,10 @@ static void usbInitProtocol5(WacomCommonPtr common, const char* id,
 	common->wcmPktLength = sizeof(struct input_event);
 	common->wcmCursorProxoutDistDefault 
 			= PROXOUT_INTUOS_DISTANCE;
+
+	/* tilt enabled */
+	common->wcmFlags |= TILT_ENABLED_FLAG;
+
 	/* reinitialize max here since 0 is for Graphire series */
 	common->wcmMaxCursorDist = 256;
 
@@ -549,6 +597,9 @@ static void usbInitProtocol4(WacomCommonPtr common, const char* id,
 	common->wcmPktLength = sizeof(struct input_event);
 	common->wcmCursorProxoutDistDefault 
 			= PROXOUT_GRAPHIRE_DISTANCE;
+
+	/* tilt disabled */
+	common->wcmFlags &= ~TILT_ENABLED_FLAG;
 }
 
 int usbWcmGetRanges(LocalDevicePtr local)
@@ -556,7 +607,8 @@ int usbWcmGetRanges(LocalDevicePtr local)
 	int nValues[5];
 	unsigned long ev[NBITS(EV_MAX)];
 	unsigned long abs[NBITS(ABS_MAX)];
-	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	WacomCommonPtr common =	priv->common;
 
 	if (ioctl(local->fd, EVIOCGBIT(0 /*EV*/, sizeof(ev)), ev) < 0)
 	{
@@ -595,12 +647,13 @@ int usbWcmGetRanges(LocalDevicePtr local)
 		ErrorF("WACOM: unable to ioctl xmax value.\n");
 		return !Success;
 	}
-	common->wcmMaxX = nValues[2];
-	if (common->wcmMaxX <= 0)
+
+	if (nValues[2] <= 0)
 	{
 		ErrorF("WACOM: xmax value is wrong.\n");
 		return !Success;
 	}
+	common->wcmMaxX = nValues[2];
 
 	/* max y */
 	if (ioctl(local->fd, EVIOCGABS(ABS_Y), nValues) < 0)
@@ -608,12 +661,48 @@ int usbWcmGetRanges(LocalDevicePtr local)
 		ErrorF("WACOM: unable to ioctl ymax value.\n");
 		return !Success;
 	}
-	common->wcmMaxY = nValues[2];
-	if (common->wcmMaxY <= 0)
+
+	if (nValues[2] <= 0)
 	{
 		ErrorF("WACOM: ymax value is wrong.\n");
 		return !Success;
 	}
+	common->wcmMaxY = nValues[2];
+
+	/* max finger strip X for tablets with Expresskeys
+	 * or max touch logical X for TabletPCs with touch */
+	if (ioctl(local->fd, EVIOCGABS(ABS_RX), nValues) == 0)
+	{
+		if (IsTouch(priv))
+			common->wcmMaxTouchX = nValues[2];
+		else
+			common->wcmMaxStripX = nValues[2];
+	}
+
+	/* max finger strip X */
+	if (ioctl(local->fd, EVIOCGABS(ABS_RY), nValues) == 0)
+	{
+		if (IsTouch(priv))
+			common->wcmMaxTouchY = nValues[2];
+		else
+			common->wcmMaxStripY = nValues[2];
+	}
+
+	if (IsTouch(priv) && common->wcmMaxX && common->wcmMaxY)
+	{
+		common->wcmTouchResolX = common->wcmMaxTouchX *
+			common->wcmResolX / common->wcmMaxX;
+		common->wcmTouchResolY = common->wcmMaxTouchY *
+			common->wcmResolY / common->wcmMaxY;
+		if (!common->wcmTouchResolX || !common->wcmTouchResolY)
+		{
+			ErrorF("WACOM: touch max value(s) was wrong MaxTouchY"
+				" = %d MaxTouchY = %d.\n", common->wcmMaxTouchX,
+				common->wcmMaxTouchY);
+			return !Success;
+		}
+
+	}
 
 	/* max z cannot be configured */
 	if (ioctl(local->fd, EVIOCGABS(ABS_PRESSURE), nValues) < 0)
@@ -621,12 +710,12 @@ int usbWcmGetRanges(LocalDevicePtr local)
 		ErrorF("WACOM: unable to ioctl press max value.\n");
 		return !Success;
 	}
-	common->wcmMaxZ = nValues[2];
-	if (common->wcmMaxZ <= 0)
+	if (nValues[2] <= 0)
 	{
 		ErrorF("WACOM: press max value is wrong.\n");
 		return !Success;
 	}
+	common->wcmMaxZ = nValues[2];
 
 	/* max distance */
 	if (ioctl(local->fd, EVIOCGABS(ABS_DISTANCE), nValues) < 0)
@@ -634,18 +723,12 @@ int usbWcmGetRanges(LocalDevicePtr local)
 		ErrorF("WACOM: unable to ioctl press max distance.\n");
 		return !Success;
 	}
-	common->wcmMaxDist = nValues[2];
-	if (common->wcmMaxDist < 0)
+	if (nValues[2] < 0)
 	{
 		ErrorF("WACOM: max distance value is wrong.\n");
 		return !Success;
 	}
-
-	/* max fingerstrip X */
-	if (ioctl(local->fd, EVIOCGABS(ABS_RX), nValues) == 0)
-		common->wcmMaxStripX = nValues[2];
-	if (ioctl(local->fd, EVIOCGABS(ABS_RY), nValues) == 0)
-		common->wcmMaxStripY = nValues[2];
+	common->wcmMaxDist = nValues[2];
 
 	return Success;
 }
@@ -662,9 +745,9 @@ static int usbDetectConfig(LocalDevicePtr local)
 
 /* This code will be used when we are ready to report valuators in tablet and tool 
  * specific form, whcih will need to clean InitValuatorAxisStruct() in xf86Wacom.c
- * and all the calls to X related to valuators, such as xf86PostButtonEvent and 
- * xf86PostButtonEvent, etc. Code under util will need to be updaed as well.
- * This will need some time. We put it in the to-do list for now.  Ping 
+ * and all the calls to X that are related to valuators, such as xf86PostButtonEvent and 
+ * xf86PostButtonEvent, etc. Code under util directory will need to be updated as well.
+ * This will take some time. We put it in the to-do list for now.  Ping 
 		unsigned long abs[NBITS(ABS_MAX)];
 		priv->naxes = 0;
 		if (ioctl(local->fd, EVIOCGBIT(EV_ABS, sizeof(abs)), abs) >= 0)
@@ -830,7 +913,12 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 		(ds->buttons | (shift)) : (ds->buttons & ~(shift))); \
 		} while (0)
 
-	DBG(6, common->debugLevel, ErrorF("usbParseChannel \n"));
+	if (common->wcmEventCnt == 1 && !common->wcmEvents->type) {
+		DBG(6, common->debugLevel, ErrorF("usbParseChannel no real events received\n"));
+		return;
+	}
+	DBG(6, common->debugLevel, ErrorF("usbParseChannel %d events received\n", common->wcmEventCnt));
+
 	/* all USB data operates from previous context except relative values*/
 	ds = &common->wcmChannel[channel].work;
 	ds->relwheel = 0;
@@ -858,12 +946,15 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 			else if (event->code == ABS_RZ) 
 				ds->rotation = event->value;
 			else if (event->code == ABS_TILT_X)
-				ds->tiltx = event->value - 64;
+				ds->tiltx = event->value - common->wcmMaxtiltX/2;
 			else if (event->code ==  ABS_TILT_Y)
-				ds->tilty = event->value - 64;
-			else if (event->code == ABS_PRESSURE)
-				ds->pressure = event->value;
-			else if (event->code == ABS_DISTANCE)
+				ds->tilty = event->value - common->wcmMaxtiltY/2;
+			else if (event->code == ABS_PRESSURE) {
+				if (ds->device_type == TOUCH_ID)
+					ds->capacity = event->value;
+				else
+					ds->pressure = event->value;
+			} else if (event->code == ABS_DISTANCE)
 				ds->distance = event->value;
 			else if (event->code == ABS_WHEEL || 
 				    event->code == ABS_Z)
@@ -889,6 +980,7 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 				(event->code == BTN_TOOL_AIRBRUSH))
 			{
 				ds->device_type = STYLUS_ID;
+				ds->device_id = STYLUS_DEVICE_ID;
 				ds->proximity = (event->value != 0);
 				DBG(6, common->debugLevel, ErrorF(
 					"USB stylus detected %x\n",
@@ -897,33 +989,47 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 			else if (event->code == BTN_TOOL_RUBBER)
 			{
 				ds->device_type = ERASER_ID;
+				ds->device_id = ERASER_DEVICE_ID;
 				ds->proximity = (event->value != 0);
 				if (ds->proximity)
 					ds->proximity = ERASER_PROX;
 				DBG(6, common->debugLevel, ErrorF(
-					"USB eraser detected %x\n",
-					event->code));
+					"USB eraser detected %x (value=%d)\n",
+					event->code, event->value));
 			}
 			else if ((event->code == BTN_TOOL_MOUSE) ||
 				(event->code == BTN_TOOL_LENS))
 			{
 				DBG(6, common->debugLevel, ErrorF(
-					"USB mouse detected %x\n",
-					event->code));
+					"USB mouse detected %x (value=%d)\n",
+					event->code, event->value));
 				ds->device_type = CURSOR_ID;
+				ds->device_id = CURSOR_DEVICE_ID;
 				ds->proximity = (event->value != 0);
 			}
 			else if (event->code == BTN_TOOL_FINGER)
 			{
 				DBG(6, common->debugLevel, ErrorF(
-					"USB Pad detected %x\n",
-					event->code));
+					"USB Pad detected %x (value=%d)\n",
+					event->code, event->value));
 				ds->device_type = PAD_ID;
+				ds->device_id = PAD_DEVICE_ID;
 				ds->proximity = (event->value != 0);
 			}
 			else if (event->code == BTN_TOUCH)
 			{
-				/* we use the pressure to determine the button 1 */
+				DBG(6, common->debugLevel, ErrorF(
+					"USB Touch detected %x (value=%d)\n",
+					event->code, event->value));
+				ds->device_type = TOUCH_ID;
+				ds->device_id = TOUCH_DEVICE_ID;
+				ds->proximity = event->value;
+				/* left button is always pressed for touch without capacity 
+				 * For touch with capacity, left button event will be decided
+				 * in wcmCommon.c by capacity threshold
+				 */
+				if (common->wcmCapacityDefault < 0)
+					MOD_BUTTONS (0, event->value);
 			}
 			else if ((event->code == BTN_STYLUS) ||
 				(event->code == BTN_MIDDLE))
@@ -953,6 +1059,10 @@ static void usbParseChannel(LocalDevicePtr local, int channel, int serial)
 		}
 	} /* next event */
 
+	/* don't send touch event when touch isn't enabled */
+	if (ds->device_type == TOUCH_ID && !common->wcmTouch)
+		return;
+
 	/* it is an out-prox when id or/and serial number is zero for kernel 2.4 */
 	if ((!ds->device_id || !ds->serial_num) && !USE_SYN_REPORTS(common))
  		ds->proximity = 0;
