@@ -20,6 +20,8 @@
  * REVISION HISTORY
  *
  * 2007-05-25 0.1 - Initial release - span off from xf86Wacom.c
+ * 2008-05-14 0.2 - Rotate through routine xf86WcmRotateScreen
+ * 2008-06-26 0.3 - Added Capacity
  */
 
 
@@ -30,6 +32,7 @@
 
 extern void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes);
 extern void xf86WcmInitialScreens(LocalDevicePtr local);
+extern void xf86WcmRotateScreen(LocalDevicePtr local, int value);
 
 /*****************************************************************************
  * xf86WcmSetPadCoreMode
@@ -145,12 +148,8 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
-	WacomDevicePtr tmppriv;
 	char st[32];
-	int oldRotation;
-	int tmpTopX, tmpTopY, tmpBottomX, tmpBottomY, oldMaxX, oldMaxY;
 	WacomToolAreaPtr area = priv->toolarea;
-	WacomToolAreaPtr tmparea;
 
 	/* We don't reset options to the values that the driver are using.  
 	 * This eliminates confusion when driver is running on default values.
@@ -279,7 +278,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		break;
 	    case XWACOM_PARAM_PRESSCURVE:
 	    {
-		if ( !IsCursor(priv) && !IsPad (priv) ) 
+		if ( !IsCursor(priv) && !IsPad (priv) && !IsTouch (priv)) 
 		{
 			char chBuf[64];
 			int x0 = (value >> 24) & 0xFF;
@@ -312,9 +311,13 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		xf86AddNewOption(local->options, "Speed", st);
 		break;
 	    case XWACOM_PARAM_ACCEL:
-		if ((value < 1) || (value > MAX_ACCEL)) return BadValue;
-		priv->accel = value-1;
-		xf86ReplaceIntOption(local->options, "Accel", priv->accel);
+		if ((value < 1) || (value > MAX_ACCEL)) 
+			return BadValue;
+		else if (priv->accel != value-1)
+		{
+			priv->accel = value-1;
+			xf86ReplaceIntOption(local->options, "Accel", priv->accel);
+		}
 		break;
 	    case XWACOM_PARAM_CLICKFORCE:
 		if ((value < 1) || (value > 21)) return BadValue;
@@ -326,8 +329,8 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 	    case XWACOM_PARAM_XYDEFAULT:
 		xf86WcmSetParam (local, XWACOM_PARAM_TOPX, 0);
 		xf86WcmSetParam (local, XWACOM_PARAM_TOPY, 0);
-		xf86WcmSetParam (local, XWACOM_PARAM_BOTTOMX, common->wcmMaxX);
-		xf86WcmSetParam (local, XWACOM_PARAM_BOTTOMY, common->wcmMaxY);
+		xf86WcmSetParam (local, XWACOM_PARAM_BOTTOMX, priv->wcmMaxX);
+		xf86WcmSetParam (local, XWACOM_PARAM_BOTTOMY, priv->wcmMaxY);
 		break;
 	    case XWACOM_PARAM_MMT:
 		if ((value != 0) && (value != 1)) 
@@ -355,6 +358,27 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 				xf86ReplaceStrOption(local->options, "TPCButton", "off");
 		}
 		break;
+	    case XWACOM_PARAM_TOUCH:
+		if ((value != 0) && (value != 1)) 
+			return BadValue;
+		else if (common->wcmTouch != value)
+		{
+			common->wcmTouch = value;
+			if (value)
+				xf86ReplaceStrOption(local->options, "Touch", "on");
+			else
+				xf86ReplaceStrOption(local->options, "Touch", "off");
+		}
+		break;
+	    case XWACOM_PARAM_CAPACITY:
+		if ((value < -1) || (value > 4)) 
+			return BadValue;
+		else if (common->wcmCapacity != value)
+		{
+			common->wcmCapacity = value;
+			xf86ReplaceIntOption(local->options, "Capacity", value);
+		}
+		break;
 	    case XWACOM_PARAM_CURSORPROX:
 		if (IsCursor (priv))
 		{
@@ -371,9 +395,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		if (value < -1 || value >= priv->numScreen) 
 			return BadValue;
 		else if (priv->screen_no != value)
-		{
 			xf86WcmChangeScreen(local, value);
-		}
 		break;
 	    case XWACOM_PARAM_TWINVIEW:
 		if (priv->twinview != value)
@@ -381,7 +403,19 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			if ((value > 2) || (value < 0) || screenInfo.numScreens != 1)
 				return BadValue;
 			priv->twinview = value;
-			xf86WcmChangeScreen(local, priv->screen_no);
+
+			/* Can not restrict the cursor to a particular screen */
+			if (!value)
+			{
+				value = -1;
+				priv->currentScreen = 0;
+				DBG(10, priv->debugLevel, ErrorF("xf86WcmSetParam(TWINVIEW) TwinView sets to "
+					"TV_NONE: cann't change screen_no. \n"));
+			}
+			else
+				value = priv->screen_no;
+
+			xf86WcmChangeScreen(local, value);
 		}
 		break;
 	    case XWACOM_PARAM_TVRESOLUTION0:
@@ -392,37 +426,50 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		else
 		{
 			int sNum = param - XWACOM_PARAM_TVRESOLUTION0;
-			int rX = 0, rY = 0;
-			if (sNum)
-			{
-				rX = priv->tvResolution[0];
-				rY = priv->tvResolution[1];
-				sNum++;
-			}
-			else
-			{
-				rX = priv->tvResolution[2];
-				rY = priv->tvResolution[3];
-			}
-			if ( priv->twinview == TV_ABOVE_BELOW )
-			{
-				rX = value & 0xffff;
-				rY += (value >> 16) & 0xffff;
-			}
-			else
-			{
-				rX += value & 0xffff;
-				rY = (value >> 16) & 0xffff;
-			}
+			int rX = value & 0xffff, rY = (value >> 16) & 0xffff;
 			if (rX > screenInfo.screens[0]->width ||
 					rY > screenInfo.screens[0]->height)
+			{
+				ErrorF("xf86WcmSetParam tvResolution out of range: " 
+					"ResX=%d ResY=%d \n", rX, rY);
 				return BadValue;
-			priv->tvResolution[sNum++] = value & 0xffff;
-			priv->tvResolution[sNum] = (value >> 16) & 0xffff;
-			xf86WcmChangeScreen(local, priv->screen_no);
+			}
+
 			DBG(10, priv->debugLevel, ErrorF("xf86WcmSetParam " 
-					"to ResX=%d ResY=%d \n",
-				value & 0xffff, (value >> 16) & 0xffff));
+				"tvResolutionX from %d to ResX=%d tvResolutionY "
+				"from %d toResY=%d \n",
+				priv->tvResolution[0], rX, priv->tvResolution[1], rY));
+
+			if ( priv->twinview == TV_ABOVE_BELOW )
+			{
+				if (sNum)
+				{
+					priv->tvResolution[1] = screenInfo.screens[0]->height - rY;
+					priv->tvResolution[2] = rX;
+					priv->tvResolution[3] = rY;
+				}
+				else
+				{
+					priv->tvResolution[0] = rX;
+					priv->tvResolution[1] = rY;
+					priv->tvResolution[3] = screenInfo.screens[0]->height - rY;
+				}
+			}
+			else
+			{
+				if (sNum)
+				{
+					priv->tvResolution[0] = screenInfo.screens[0]->width - rX;
+					priv->tvResolution[2] = rX;
+					priv->tvResolution[3] = rY;
+				}
+				else
+				{
+					priv->tvResolution[0] = rX;
+					priv->tvResolution[1] = rY;
+					priv->tvResolution[2] = screenInfo.screens[0]->width - rX;
+				}
+			}
 		}
 		break;
 	    }
@@ -442,106 +489,10 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		break;
 	   case XWACOM_PARAM_ROTATE:
 		if ((value < 0) || (value > 3)) return BadValue;
-		switch(value) {
-		  case ROTATE_NONE:
-			xf86ReplaceStrOption(local->options, "Rotate", "NONE");
-			break;
-		  case ROTATE_CW:
-			xf86ReplaceStrOption(local->options, "Rotate", "CW");
-			break;
-		  case ROTATE_CCW:
-			xf86ReplaceStrOption(local->options, "Rotate", "CCW");
-			break;
-		  case ROTATE_HALF:
-			xf86ReplaceStrOption(local->options, "Rotate", "HALF");
-			break;
-		  default:
-			return BadValue;
-		}
-		oldRotation = common->wcmRotate;
-		oldMaxX = common->wcmMaxX;
-		oldMaxY = common->wcmMaxY;
-		common->wcmRotate = value;
-		if (((oldRotation == ROTATE_NONE || oldRotation == ROTATE_HALF) && 
-			(value == ROTATE_CW || value == ROTATE_CCW)) ||
-		     ((oldRotation == ROTATE_CW || oldRotation == ROTATE_CCW) 
-			&& (value == ROTATE_NONE || value == ROTATE_HALF)))
-		{
-		    common->wcmMaxX = oldMaxY;
-		    common->wcmMaxY = oldMaxX;
-		}
-
-		/* rotate all devices at once! else they get misaligned */
-		for (tmppriv = common->wcmDevices; tmppriv; tmppriv = tmppriv->next)
-		{
-		    tmparea = tmppriv->toolarea;
-		    /* recover the unrotated xy-rectangles */
-		    switch (oldRotation) {
-		      case ROTATE_CW:
-			tmpTopX = oldMaxY - tmppriv->bottomY;
-			tmpBottomX = oldMaxY - tmppriv->topY;
-			tmpTopY = tmppriv->topX;
-			tmpBottomY = tmppriv->bottomX;
-			break;
-		      case ROTATE_CCW:
-			tmpTopX = tmppriv->topY;
-			tmpBottomX = tmppriv->bottomY;
-			tmpTopY = oldMaxX - tmppriv->bottomX;
-			tmpBottomY = oldMaxX - tmppriv->topX;
-			break;
-		      case ROTATE_HALF:
-			tmpTopX = oldMaxX - tmppriv->bottomX;
-			tmpBottomX = oldMaxX - tmppriv->topX;
-			tmpTopY = oldMaxY - tmppriv->bottomY;
-			tmpBottomY = oldMaxY - tmppriv->topY;
-			break;
-		      default: /* ROTATE_NONE */
-			tmpTopX = tmppriv->topX;
-			tmpBottomX = tmppriv->bottomX;
-			tmpTopY = tmppriv->topY;
-			tmpBottomY = tmppriv->bottomY;
-			break;
-		    } 
-		    /* and rotate them back */
-		    switch (value) {
-		      case ROTATE_CW:
-			tmparea->topX = tmppriv->topX = tmpTopY;
-			tmparea->bottomX = tmppriv->bottomX = tmpBottomY;
-			tmparea->topY = tmppriv->topY = common->wcmMaxY - tmpBottomX;
-			tmparea->bottomY = tmppriv->bottomY = common->wcmMaxY - tmpTopX;
-			break;
-		      case ROTATE_CCW:
-			tmparea->topX = tmppriv->topX = common->wcmMaxX - tmpBottomY;
-			tmparea->bottomX = tmppriv->bottomX = common->wcmMaxX - tmpTopY;
-			tmparea->topY = tmppriv->topY = tmpTopX;
-			tmparea->bottomY = tmppriv->bottomY = tmpBottomX;
-			break;
-		      case ROTATE_HALF:
-			tmparea->topX = tmppriv->topX = common->wcmMaxX - tmpBottomX;
-			tmparea->bottomX = tmppriv->bottomX = common->wcmMaxX - tmpTopX;
-			tmparea->topY = tmppriv->topY= common->wcmMaxY - tmpBottomY;
-			tmparea->bottomY = tmppriv->bottomY = common->wcmMaxY - tmpTopY;
-			break;
-		      default: /* ROTATE_NONE */
-			tmparea->topX = tmppriv->topX = tmpTopX;
-			tmparea->bottomX = tmppriv->bottomX = tmpBottomX;
-			tmparea->topY = tmppriv->topY = tmpTopY;
-			tmparea->bottomY = tmppriv->bottomY = tmpBottomY;
-			break;
-		    }
-
-		    xf86WcmInitialScreens(tmppriv->local);
-		    xf86WcmMappingFactor(tmppriv->local);
-		    xf86WcmInitialCoordinates(tmppriv->local, 0);
-		    xf86WcmInitialCoordinates(tmppriv->local, 1);
-
-		    xf86ReplaceIntOption(tmppriv->local->options, "TopX", tmppriv->topX);
-		    xf86ReplaceIntOption(tmppriv->local->options, "TopY", tmppriv->topY);
-		    xf86ReplaceIntOption(tmppriv->local->options, "BottomX", tmppriv->bottomX);
-		    xf86ReplaceIntOption(tmppriv->local->options, "BottomY", tmppriv->bottomY);
-		}
+		if (common->wcmRotate != value)
+			xf86WcmRotateScreen(local, value);
 		break;
-	      default:
+	   default:
 		DBG(10, priv->debugLevel, ErrorF("xf86WcmSetParam invalid param %d\n",param));
 		return BadMatch;
 	}
@@ -825,7 +776,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_RAWSAMPLE:
 		return common->wcmRawSample;
 	    case XWACOM_PARAM_PRESSCURVE:
-		if (!IsCursor (priv) && !IsPad (priv))
+		if (!IsCursor (priv) && !IsPad (priv) && !IsTouch (priv))
 			return (priv->nPressCtrl [0] << 24) |
 			       (priv->nPressCtrl [1] << 16) |
 			       (priv->nPressCtrl [2] << 8) |
@@ -850,6 +801,10 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 		return priv->wcmMMonitor;
 	    case XWACOM_PARAM_TPCBUTTON:
 		return common->wcmTPCButton;
+	    case XWACOM_PARAM_TOUCH:
+		return common->wcmTouch;
+	    case XWACOM_PARAM_CAPACITY:
+		return common->wcmCapacity;
 	    case XWACOM_PARAM_CURSORPROX:
 		if (IsCursor (priv))
 			return common->wcmCursorProxoutDist;
@@ -877,6 +832,12 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 		return priv->screen_no;
 	    case XWACOM_PARAM_NUMSCREEN:
 		return priv->numScreen;
+	    case XWACOM_PARAM_XSCALING:
+#ifdef WCM_XORG_TABLET_SCALING
+		return 1;
+#else
+		return 0;
+#endif
 	}
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmGetParam invalid param %d\n", param));
 	return -1;
@@ -981,9 +942,9 @@ static int xf86WcmGetDefaultParam(LocalDevicePtr local, int param)
 	case XWACOM_PARAM_TOPY:
 		return 0;
 	case XWACOM_PARAM_BOTTOMX:
-		return common->wcmMaxX;
+		return priv->wcmMaxX;
 	case XWACOM_PARAM_BOTTOMY:
-		return common->wcmMaxY;		
+		return priv->wcmMaxY;		
 	case XWACOM_PARAM_BUTTON1:
 	case XWACOM_PARAM_BUTTON2:
 	case XWACOM_PARAM_BUTTON3:
@@ -991,9 +952,9 @@ static int xf86WcmGetDefaultParam(LocalDevicePtr local, int param)
 	case XWACOM_PARAM_BUTTON5:
 		return (param - XWACOM_PARAM_BUTTON1 + 1);
 	case XWACOM_PARAM_MODE:
-                if (IsCursor(priv) || (IsPad(priv) && (priv->flags & COREEVENT_FLAG)))
+		if (IsCursor(priv) || (IsPad(priv) && (priv->flags & COREEVENT_FLAG)))
 			return 0;
-                else
+		else
 			return 1;
 	case XWACOM_PARAM_RELWUP:
 	case XWACOM_PARAM_ABSWUP:
@@ -1015,8 +976,12 @@ static int xf86WcmGetDefaultParam(LocalDevicePtr local, int param)
 		return 1;
 	case XWACOM_PARAM_TPCBUTTON:
 		return common->wcmTPCButtonDefault;
+	case XWACOM_PARAM_TOUCH:
+		return common->wcmTouchDefault;
+	case XWACOM_PARAM_CAPACITY:
+		return common->wcmCapacityDefault;
 	case XWACOM_PARAM_PRESSCURVE:
-		if (!IsCursor (priv) && !IsPad (priv))
+		if (!IsCursor (priv) && !IsPad (priv) && !IsTouch (priv))
 			return (0 << 24) | (0 << 16) | (100 << 8) | 100;
 		return -1;
 	case XWACOM_PARAM_SUPPRESS:
